@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2020 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
+ * Copyright (c) 2011-2021 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -65,7 +65,7 @@ static void
 version(void)
 {
     fprintf(stderr, "fpart v" FPART_VERSION "\n"
-        "Copyright (c) 2011-2020 "
+        "Copyright (c) 2011-2021 "
         "Ganael LAPLANCHE <ganael.laplanche@martymac.org>\n"
         "WWW: http://contribs.martymac.org\n");
     fprintf(stderr, "Build options: debug=");
@@ -260,13 +260,12 @@ handle_options(struct program_options *options, int *argcp, char ***argvp)
     int ch;
     while((ch = getopt(*argcp, *argvp,
 #if defined(_HAS_FNM_CASEFOLD)
-        "?hVn:f:s:i:ao:0evlby:Y:x:X:zd:DELw:W:p:q:r:"
+        "hVn:f:s:i:ao:0evlby:Y:x:X:zd:DELw:W:p:q:r:"
 #else
-        "?hVn:f:s:i:ao:0evlby:x:zd:DELw:W:p:q:r:"
+        "hVn:f:s:i:ao:0evlby:x:zd:DELw:W:p:q:r:"
 #endif
         )) != -1) {
         switch(ch) {
-            case '?':
             case 'h':
                 return (FPART_OPTS_USAGE | FPART_OPTS_OK | FPART_OPTS_EXIT);
             case 'V':
@@ -485,6 +484,9 @@ handle_options(struct program_options *options, int *argcp, char ***argvp)
                 options->round_size = (fsize_t)round_size;
                 break;
             }
+            case '?':
+            default:
+                return (FPART_OPTS_USAGE | FPART_OPTS_NOK | FPART_OPTS_EXIT);
         }
     }
     *argcp -= optind;
