@@ -1,4 +1,6 @@
 /*-
+ * SPDX-License-Identifier: BSD-3-Clause
+ *
  * Copyright (c) 1990, 1993, 1994
  *	The Regents of the University of California.  All rights reserved.
  *
@@ -38,15 +40,10 @@
  *
  */
 
-#if 0
-#if defined(LIBC_SCCS) && !defined(lint)
-static char sccsid[] = "@(#)fts.c	8.6 (Berkeley) 8/14/94";
-#endif /* LIBC_SCCS and not lint */
-#endif
-
 #if defined(__FreeBSD__)
 #include <sys/cdefs.h>
-__FBSDID("$FreeBSD: head/lib/libc/gen/fts.c 318913 2017-05-26 01:14:58Z pfg $");
+__SCCSID("@(#)fts.c    8.6 (Berkeley) 8/14/94");
+__FBSDID("$FreeBSD: head/lib/libc/gen/fts.c 345552 2019-03-26 19:35:41Z emaste $");
 
 #include "/usr/src/lib/libc/include/namespace.h"
 #else
@@ -54,7 +51,8 @@ __FBSDID("$FreeBSD: head/lib/libc/gen/fts.c 318913 2017-05-26 01:14:58Z pfg $");
 #define _close close
 #define _fstat fstat
 #define _dirfd dirfd
-#endif
+#endif /* defined(__FreeBSD__) */
+
 #include <sys/param.h>
 #include <sys/mount.h>
 #include <sys/stat.h>
@@ -67,8 +65,19 @@ __FBSDID("$FreeBSD: head/lib/libc/gen/fts.c 318913 2017-05-26 01:14:58Z pfg $");
 #if defined(__linux__)
 #include <sys/vfs.h>
 #include <sys/types.h>
+#endif /* defined(__linux__) */
+#endif /* defined(__sun) || defined(__sun__) */
+
+#include <dirent.h>
+#include <errno.h>
 #include <fcntl.h>
-#endif
+#include "fts.h"
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
+#if defined(__FreeBSD__)
+#include "/usr/src/lib/libc/include/un-namespace.h"
+#include "/usr/src/lib/libc/gen/gen-private.h"
 #endif
 
 #if defined(O_CLOEXEC)
@@ -89,18 +98,6 @@ __FBSDID("$FreeBSD: head/lib/libc/gen/fts.c 318913 2017-05-26 01:14:58Z pfg $");
 #endif
 #endif
 
-#include <dirent.h>
-#include <errno.h>
-#include <fcntl.h>
-#include "fts.h"
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-#if defined(__FreeBSD__)
-#include "/usr/src/lib/libc/include/un-namespace.h"
-#include "/usr/src/lib/libc/gen/gen-private.h"
-#endif
-
 #if defined(__sun) || defined(__sun__) || defined(__linux__)
 void *
 reallocf(void *ptr, size_t size)
@@ -723,9 +720,9 @@ fts_build(FTS *sp, int type)
 	 */
 #ifdef FTS_WHITEOUT
 	if (ISSET(FTS_WHITEOUT))
-		oflag = DTF_NODUP | DTF_REWIND;
+		oflag = DTF_NODUP;
 	else
-		oflag = DTF_HIDEW | DTF_NODUP | DTF_REWIND;
+		oflag = DTF_HIDEW | DTF_NODUP;
 #else
 #define __opendir2(path, flag) opendir(path)
 #endif
@@ -1219,11 +1216,20 @@ fts_safe_changedir(FTS *sp, FTSENT *p, int fd, char *path)
 		ret = -1;
 		goto bail;
 	}
+/*
+ * On Darwin, this check causes problems with automounts and firmlinks
+ * so we just disable it.
+ * See: /usr/share/firmlinks for a list of problematic directories
+ * and https://opensource.apple.com/source/Libc/Libc-1439.40.11/gen/fts.c.auto.html
+ * (function fts_safe_changedir()) for more details.
+ */
+#if !defined(__APPLE__)
 	if (p->fts_dev != sb.st_dev || p->fts_ino != sb.st_ino) {
 		errno = ENOENT;		/* disinformation */
 		ret = -1;
 		goto bail;
 	}
+#endif /* !defined(__APPLE__) */
 	ret = fchdir(newfd);
 bail:
 	oerrno = errno;
